@@ -204,6 +204,7 @@ impl Info {
 
 /// Show the flamegraph.
 pub fn ui(ui: &mut egui::Ui, options: &mut Options, frames: &SelectedFrames) {
+    puffin::profile_function!();
     let mut reset_view = false;
 
     let num_frames = frames.frames.len();
@@ -251,7 +252,8 @@ pub fn ui(ui: &mut egui::Ui, options: &mut Options, frames: &SelectedFrames) {
     Frame::dark_canvas(ui.style()).show(ui, |ui| {
         let available_height = ui.max_rect().bottom() - ui.min_rect().bottom();
         ScrollArea::vertical().show(ui, |ui| {
-            let canvas = ui.available_rect_before_wrap();
+            let mut canvas = ui.available_rect_before_wrap();
+            canvas.max.y = f32::INFINITY;
             let response = ui.interact(canvas, ui.id(), Sense::click_and_drag());
 
             let (min_ns, max_ns) = if options.merge_scopes {
